from types import *
from . import (
    pdfdoc as pdfdoc,
    pdfgeom as pdfgeom,
    pdfmetrics as pdfmetrics,
    pdfutils as pdfutils,
)
from _typeshed import Incomplete
from math import ceil as ceil

class PDFError(ValueError): ...

FILL_EVEN_ODD: int
FILL_NON_ZERO: int
PATH_OPS: Incomplete
close: str
newpath: str
stroke: str
closeStroke: str
nzFill: str
eoFill: str
fillStroke: str
closeFillStroke: str
eoFillStroke: str
closeEoFillStroke: str

class Canvas:
    _filename: Incomplete
    _doc: Incomplete
    _pagesize: Incomplete
    _currentPageHasImages: int
    _pageTransitionString: str
    _pageCompression: int
    _pageNumber: int
    _code: Incomplete
    bottomup: Incomplete
    _preamble: str
    _x: int
    _y: int
    _fontname: str
    _fontsize: int
    _textMode: int
    _leading: float
    _currentMatrix: Incomplete
    _fillMode: int
    _charSpace: int
    _wordSpace: int
    _horizScale: int
    _textRenderMode: int
    _rise: int
    _textLineMatrix: Incomplete
    _textMatrix: Incomplete
    _lineCap: int
    _lineJoin: int
    _lineDash: Incomplete
    _lineWidth: int
    _mitreLimit: int
    _fillColorRGB: Incomplete
    _strokeColorRGB: Incomplete
    def __init__(self, filename, pagesize=..., bottomup: int = ...) -> None: ...
    def _escape(self, s): ...
    def setAuthor(self, author) -> None: ...
    def setTitle(self, title) -> None: ...
    def setSubject(self, subject) -> None: ...
    def pageHasData(self): ...
    def showPage(self) -> None: ...
    def getPageNumber(self): ...
    def save(
        self, filename: Incomplete | None = ..., fileobj: Incomplete | None = ...
    ) -> None: ...
    def setPageSize(self, size) -> None: ...
    def addLiteral(self, s, escaped: int = ...) -> None: ...
    def transform(self, a, b, c, d, e, f) -> None: ...
    def translate(self, dx, dy) -> None: ...
    def scale(self, x, y) -> None: ...
    def rotate(self, theta) -> None: ...
    def skew(self, alpha, beta) -> None: ...
    def saveState(self) -> None: ...
    def restoreState(self) -> None: ...
    def line(self, x1, y1, x2, y2) -> None: ...
    def lines(self, linelist) -> None: ...
    def grid(self, xlist, ylist) -> None: ...
    def bezier(self, x1, y1, x2, y2, x3, y3, x4, y4) -> None: ...
    def arc(self, x1, y1, x2, y2, startAng: int = ..., extent: int = ...) -> None: ...
    def rect(self, x, y, width, height, stroke: int = ..., fill: int = ...) -> None: ...
    def ellipse(self, x1, y1, x2, y2, stroke: int = ..., fill: int = ...) -> None: ...
    def wedge(
        self, x1, y1, x2, y2, startAng, extent, stroke: int = ..., fill: int = ...
    ) -> None: ...
    def circle(self, x_cen, y_cen, r, stroke: int = ..., fill: int = ...) -> None: ...
    def roundRect(
        self, x, y, width, height, radius, stroke: int = ..., fill: int = ...
    ) -> None: ...
    def drawString(self, x, y, text) -> None: ...
    def drawRightString(self, x, y, text) -> None: ...
    def drawCentredString(self, x, y, text) -> None: ...
    def getAvailableFonts(self): ...
    def setFont(self, psfontname, size, leading: Incomplete | None = ...) -> None: ...
    def stringWidth(self, text, fontname, fontsize): ...
    def setLineWidth(self, width) -> None: ...
    def setLineCap(self, mode) -> None: ...
    def setLineJoin(self, mode) -> None: ...
    _miterLimit: Incomplete
    def setMiterLimit(self, limit) -> None: ...
    def setDash(self, array=..., phase: int = ...) -> None: ...
    def setFillColorRGB(self, r, g, b) -> None: ...
    def setStrokeColorRGB(self, r, g, b) -> None: ...
    def beginPath(self): ...
    def drawPath(self, aPath, stroke: int = ..., fill: int = ...) -> None: ...
    def clipPath(self, aPath, stroke: int = ..., fill: int = ...) -> None: ...
    def beginText(self, x: int = ..., y: int = ...): ...
    def drawText(self, aTextObject) -> None: ...
    def drawInlineImage(
        self,
        image,
        x,
        y,
        width: Incomplete | None = ...,
        height: Incomplete | None = ...,
    ) -> None: ...
    def readJPEGInfo(self, image): ...
    def setPageCompression(self, onoff: int = ...) -> None: ...
    def setPageTransition(
        self,
        effectname: Incomplete | None = ...,
        duration: int = ...,
        direction: int = ...,
        dimension: str = ...,
        motion: str = ...,
    ) -> None: ...

class PDFPathObject:
    _code: Incomplete
    def __init__(self) -> None: ...
    def getCode(self): ...
    def moveTo(self, x, y) -> None: ...
    def lineTo(self, x, y) -> None: ...
    def curveTo(self, x1, y1, x2, y2, x3, y3) -> None: ...
    def arc(self, x1, y1, x2, y2, startAng: int = ..., extent: int = ...) -> None: ...
    def arcTo(self, x1, y1, x2, y2, startAng: int = ..., extent: int = ...) -> None: ...
    def rect(self, x, y, width, height) -> None: ...
    def ellipse(self, x, y, width, height) -> None: ...
    def circle(self, x_cen, y_cen, r) -> None: ...
    def close(self) -> None: ...

class PDFTextObject:
    _code: Incomplete
    _canvas: Incomplete
    _fontname: Incomplete
    _fontsize: Incomplete
    _leading: Incomplete
    def __init__(self, canvas, x: int = ..., y: int = ...) -> None: ...
    def getCode(self): ...
    _x: Incomplete
    _y: Incomplete
    _x0: Incomplete
    def setTextOrigin(self, x, y) -> None: ...
    def setTextTransform(self, a, b, c, d, e, f) -> None: ...
    def moveCursor(self, dx, dy) -> None: ...
    def getCursor(self): ...
    def getX(self): ...
    def getY(self): ...
    def setFont(self, psfontname, size, leading: Incomplete | None = ...) -> None: ...
    _charSpace: Incomplete
    def setCharSpace(self, charSpace) -> None: ...
    _wordSpace: Incomplete
    def setWordSpace(self, wordSpace) -> None: ...
    _horizScale: Incomplete
    def setHorizScale(self, horizScale) -> None: ...
    def setLeading(self, leading) -> None: ...
    _textRenderMode: Incomplete
    def setTextRenderMode(self, mode) -> None: ...
    _rise: Incomplete
    def setRise(self, rise) -> None: ...
    _strokeColorRGB: Incomplete
    def setStrokeColorRGB(self, r, g, b) -> None: ...
    _fillColorRGB: Incomplete
    def setFillColorRGB(self, r, g, b) -> None: ...
    def textOut(self, text) -> None: ...
    def textLine(self, text: str = ...) -> None: ...
    def textLines(self, stuff, trim: int = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.VLib.Filter import FilterNode as FilterNode

class DupeFilter(FilterNode):
    _smisSeen: Incomplete
    def __init__(self, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def filter(self, cmpd): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from typing import Any, ClassVar

class EnumeratorType:
    LinkNode: ClassVar[EnumeratorType] = ...
    PositionVariation: ClassVar[EnumeratorType] = ...
    RepeatUnit: ClassVar[EnumeratorType] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class MolEnumeratorParams:
    __instance_size__: ClassVar[int] = ...
    doRandom: Any
    maxToEnumerate: Any
    randomSeed: Any
    sanitize: Any
    def __init__(self, Any): ...
    @classmethod
    def SetEnumerationOperator(cls, RDKit, Any) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def Enumerate(RDKit) -> Any: ...



from _typeshed import Incomplete

class ExcludedVolume:
    index: Incomplete
    featInfo: Incomplete
    exclusionDist: Incomplete
    pos: Incomplete
    def __init__(
        self, featInfo, index: int = ..., exclusionDist: float = ...
    ) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit import Chem as Chem
from rdkit.Chem import (
    ChemicalFeatures as ChemicalFeatures,
    ChemicalForceFields as ChemicalForceFields,
)
from rdkit.Chem.Pharm3D import ExcludedVolume as ExcludedVolume
from rdkit.ML.Data import Stats as Stats

_times: Incomplete
logger: Incomplete
defaultFeatLength: float

def GetAtomHeavyNeighbors(atom): ...
def ReplaceGroup(
    match, bounds, slop: float = ..., useDirs: bool = ..., dirLength=...
): ...
def EmbedMol(
    mol,
    bm,
    atomMatch: Incomplete | None = ...,
    weight: float = ...,
    randomSeed: int = ...,
    excludedVolumes: Incomplete | None = ...,
) -> None: ...
def AddExcludedVolumes(bm, excludedVolumes, smoothIt: bool = ...): ...
def UpdatePharmacophoreBounds(
    bm,
    atomMatch,
    pcophore,
    useDirs: bool = ...,
    dirLength=...,
    mol: Incomplete | None = ...,
): ...
def EmbedPharmacophore(
    mol,
    atomMatch,
    pcophore,
    randomSeed: int = ...,
    count: int = ...,
    smoothFirst: bool = ...,
    silent: bool = ...,
    bounds: Incomplete | None = ...,
    excludedVolumes: Incomplete | None = ...,
    targetNumber: int = ...,
    useDirs: bool = ...,
): ...
def isNaN(v): ...
def OptimizeMol(
    mol,
    bm,
    atomMatches: Incomplete | None = ...,
    excludedVolumes: Incomplete | None = ...,
    forceConstant: float = ...,
    maxPasses: int = ...,
    verbose: bool = ...,
): ...
def EmbedOne(
    mol, name, match, pcophore, count: int = ..., silent: int = ..., **kwargs
): ...
def MatchPharmacophoreToMol(mol, featFactory, pcophore): ...
def _getFeatDict(mol, featFactory, features): ...
def MatchFeatsToMol(mol, featFactory, features): ...
def CombiEnum(sequence) -> Generator[Incomplete, None, None]: ...
def DownsampleBoundsMatrix(bm, indices, maxThresh: float = ...): ...
def CoarseScreenPharmacophore(atomMatch, bounds, pcophore, verbose: bool = ...): ...
def Check2DBounds(atomMatch, mol, pcophore): ...
def _checkMatch(match, mol, bounds, pcophore, use2DLimits): ...
def ConstrainedEnum(
    matches, mol, pcophore, bounds, use2DLimits: bool = ..., index: int = ..., soFar=...
) -> Generator[Incomplete, None, None]: ...
def MatchPharmacophore(
    matches,
    bounds,
    pcophore,
    useDownsampling: bool = ...,
    use2DLimits: bool = ...,
    mol: Incomplete | None = ...,
    excludedVolumes: Incomplete | None = ...,
    useDirs: bool = ...,
): ...
def GetAllPharmacophoreMatches(
    matches,
    bounds,
    pcophore,
    useDownsampling: int = ...,
    progressCallback: Incomplete | None = ...,
    use2DLimits: bool = ...,
    mol: Incomplete | None = ...,
    verbose: bool = ...,
): ...
def ComputeChiralVolume(mol, centerIdx, confId: int = ...): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete

def setDescriptorVersion(version: str = ...): ...

class VectorDescriptorNamespace(dict):
    def __init__(self, **kwargs) -> None: ...

class VectorDescriptorWrapper:
    func: Incomplete
    names: Incomplete
    func_key: Incomplete
    namespace: Incomplete
    def __init__(self, func, names, version, namespace) -> None: ...
    def _get_key(self, index): ...
    def call_desc(self, mol, index): ...



from _typeshed import Incomplete
from rdkit.ML.DecTree import DecTree as DecTree, Tree as Tree

class QuantTreeNode(DecTree.DecTreeNode):
    qBounds: Incomplete
    nBounds: int
    def __init__(self, *args, **kwargs) -> None: ...
    def ClassifyExample(self, example, appendExamples: int = ...): ...
    def SetQuantBounds(self, qBounds) -> None: ...
    def GetQuantBounds(self): ...
    def __cmp__(self, other): ...
    def __lt__(self, other): ...
    def __eq__(self, other): ...
    def __str__(self): ...



from _typeshed import Incomplete

hascEntropy: int
_log2: Incomplete

def PyInfoEntropy(results): ...
def PyInfoGain(varMat): ...

InfoEntropy: Incomplete
InfoGain: Incomplete
InfoEntropy = PyInfoEntropy
InfoGain = PyInfoGain



from typing import Any, ClassVar

class JSONParseParameters:
    __instance_size__: ClassVar[int] = ...
    parseConformers: Any
    parseProperties: Any
    setAromaticBonds: Any
    strictValenceCheck: Any
    useHCounts: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def JSONToMols(*args, **kwargs) -> Any: ...
def MolToJSON(RDKit) -> Any: ...
def MolsToJSON(boost) -> Any: ...



from _typeshed import Incomplete
from rdkit.ML.Data import Quantize as Quantize
from rdkit.ML.DecTree import ID3 as ID3, QuantTree as QuantTree
from rdkit.ML.InfoTheory import entropy as entropy

def FindBest(
    resCodes,
    examples,
    nBoundsPerVar,
    nPossibleRes,
    nPossibleVals,
    attrs,
    exIndices: Incomplete | None = ...,
    **kwargs
): ...
def BuildQuantTree(
    examples,
    target,
    attrs,
    nPossibleVals,
    nBoundsPerVar,
    depth: int = ...,
    maxDepth: int = ...,
    exIndices: Incomplete | None = ...,
    **kwargs
): ...
def QuantTreeBoot(
    examples,
    attrs,
    nPossibleVals,
    nBoundsPerVar,
    initialVar: Incomplete | None = ...,
    maxDepth: int = ...,
    **kwargs
): ...
def TestTree() -> None: ...
def TestQuantTree() -> None: ...
def TestQuantTree2() -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

class FragmentMatcher:
    _onPatt: Incomplete
    _offPatts: Incomplete
    def __init__(self) -> None: ...
    def AddExclusion(self, sma) -> None: ...
    def Init(self, sma) -> None: ...
    def GetSMARTS(self) -> None: ...
    def GetExclusionSMARTS(self) -> None: ...
    def HasMatch(self, mol): ...
    def GetMatch(self, mol): ...
    def GetMatches(self, mol, uniquify: int = ...): ...
    def GetBond(self, idx): ...

def _test(): ...



from _typeshed import Incomplete
from rdkit.ML.DecTree import Tree as Tree

class DecTreeNode(Tree.TreeNode):
    examples: Incomplete
    badExamples: Incomplete
    trainingExamples: Incomplete
    testExamples: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def ClassifyExample(self, example, appendExamples: int = ...): ...
    def AddChild(
        self,
        name,
        label: Incomplete | None = ...,
        data: Incomplete | None = ...,
        isTerminal: int = ...,
    ): ...
    def GetExamples(self): ...
    def SetExamples(self, examples) -> None: ...
    def GetBadExamples(self): ...
    def SetBadExamples(self, examples) -> None: ...
    def GetTrainingExamples(self): ...
    def SetTrainingExamples(self, examples) -> None: ...
    def GetTestExamples(self): ...
    def SetTestExamples(self, examples) -> None: ...
    def ClearExamples(self) -> None: ...



from typing import Any

class O3A:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def Align(cls, RDKit) -> Any: ...
    @classmethod
    def Matches(cls, RDKit) -> Any: ...
    @classmethod
    def Score(cls, RDKit) -> Any: ...
    @classmethod
    def Trans(cls, RDKit) -> Any: ...
    @classmethod
    def Weights(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def AlignMol(*args, **kwargs) -> Any: ...
def AlignMolConformers(*args, **kwargs) -> Any: ...
def CalcRMS(*args, **kwargs) -> Any: ...
def GetAlignmentTransform(*args, **kwargs) -> Any: ...
def GetBestRMS(*args, **kwargs) -> Any: ...
def GetCrippenO3A(*args, **kwargs) -> Any: ...
def GetCrippenO3AForProbeConfs(*args, **kwargs) -> Any: ...
def GetO3A(*args, **kwargs) -> Any: ...
def GetO3AForProbeConfs(*args, **kwargs) -> Any: ...
def RandomTransform(RDKit) -> Any: ...



from typing import Any, ClassVar

from typing import overload

class CachedMolHolder(MolHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddBinary(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class CachedSmilesMolHolder(MolHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddSmiles(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class CachedTrustedSmilesMolHolder(MolHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddSmiles(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FPHolderBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddFingerprint(cls, RDKit, ExplicitBitVect) -> Any: ...
    @classmethod
    def AddMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFingerprint(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def MakeFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def PassesFilter(cls, RDKit, unsignedint, ExplicitBitVect) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class KeyFromPropHolder(KeyHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetPropName(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class KeyHolderBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddKey(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def AddMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetKey(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetKeys(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolHolder(MolHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolHolderBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMol(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class PatternHolder(FPHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SubstructLibrary:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def CountMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFpHolder(cls, RDKit) -> Any: ...
    @classmethod
    def GetKeyHolder(cls, RDKit) -> Any: ...
    @classmethod
    def GetMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMol(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetMolHolder(cls, RDKit) -> Any: ...
    @classmethod
    def GetSearchOrder(cls, RDKit) -> Any: ...
    @classmethod
    def HasMatch(cls, *args, **kwargs) -> Any: ...
    @overload
    @classmethod
    def InitFromStream(cls, stream) -> Any: ...
    @overload
    @classmethod
    def InitFromStream(cls, f) -> Any: ...
    @overload
    @classmethod
    def InitFromStream(cls, RDKit, boost) -> Any: ...
    @classmethod
    def Serialize(cls, RDKit) -> Any: ...
    @classmethod
    def SetSearchOrder(cls, RDKit, boost) -> Any: ...
    @overload
    @classmethod
    def ToStream(cls, stream) -> Any: ...
    @overload
    @classmethod
    def ToStream(cls, RDKit, boost) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDKit) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class TautomerPatternHolder(FPHolderBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

@overload
def AddPatterns(RDKit) -> Any: ...
@overload
def AddPatterns(RDKit, boost) -> Any: ...
def SubstructLibraryCanSerialize(*args, **kwargs) -> Any: ...



from rdkit import Chem as Chem

class PropertyMol(Chem.Mol):
    __getstate_manages_dict__: bool
    def __init__(self, mol) -> None: ...
    def __getstate__(self): ...
    def __setstate__(self, stateD) -> None: ...

def _test(): ...



from rdkit import DataStructs as DataStructs
from rdkit.Chem.Fingerprints import (
    FingerprintMols as FingerprintMols,
    MolSimilarity as MolSimilarity,
)
from rdkit.ML.Cluster import Murtagh as Murtagh

message = FingerprintMols.message
error = FingerprintMols.error

def GetDistanceMatrix(data, metric, isSimilarity: int = ...): ...
def ClusterPoints(
    data,
    metric,
    algorithmId,
    haveLabels: bool = ...,
    haveActs: bool = ...,
    returnDistances: bool = ...,
): ...
def ClusterFromDetails(details): ...

_usageDoc: str



from typing import Any, Optional

from rdkit.Chem.rdchem import QueryAtom

def AAtomQueryAtom(*args, **kwargs) -> Any: ...
def AHAtomQueryAtom(*args, **kwargs) -> Any: ...
def AtomNumEqualsQueryAtom(*args, **kwargs) -> Any: ...
def AtomNumGreaterQueryAtom(*args, **kwargs) -> Any: ...
def AtomNumLessQueryAtom(*args, **kwargs) -> Any: ...
def ExplicitDegreeEqualsQueryAtom(*args, **kwargs) -> Any: ...
def ExplicitDegreeGreaterQueryAtom(*args, **kwargs) -> Any: ...
def ExplicitDegreeLessQueryAtom(*args, **kwargs) -> Any: ...
def ExplicitValenceEqualsQueryAtom(*args, **kwargs) -> Any: ...
def ExplicitValenceGreaterQueryAtom(*args, **kwargs) -> Any: ...
def ExplicitValenceLessQueryAtom(*args, **kwargs) -> Any: ...
def FormalChargeEqualsQueryAtom(*args, **kwargs) -> Any: ...
def FormalChargeGreaterQueryAtom(*args, **kwargs) -> Any: ...
def FormalChargeLessQueryAtom(*args, **kwargs) -> Any: ...
def HCountEqualsQueryAtom(*args, **kwargs) -> Any: ...
def HCountGreaterQueryAtom(*args, **kwargs) -> Any: ...
def HCountLessQueryAtom(*args, **kwargs) -> Any: ...
def HasBitVectPropWithValueQueryAtom(*args, **kwargs) -> Any: ...
def HasBoolPropWithValueQueryAtom(*args, **kwargs) -> Any: ...
def HasBoolPropWithValueQueryBond(*args, **kwargs) -> Any: ...
def HasChiralTagQueryAtom(*args, **kwargs) -> Any: ...
def HasDoublePropWithValueQueryAtom(*args, **kwargs) -> Any: ...
def HasDoublePropWithValueQueryBond(*args, **kwargs) -> Any: ...
def HasIntPropWithValueQueryAtom(
    propname: str,
    val: int,
    negate: Optional[bool] = False,
    tolerance: Optional[int] = 0,
) -> QueryAtom: ...
def HasIntPropWithValueQueryBond(*args, **kwargs) -> Any: ...
def HasPropQueryAtom(*args, **kwargs) -> Any: ...
def HasPropQueryBond(*args, **kwargs) -> Any: ...
def HasStringPropWithValueQueryAtom(*args, **kwargs) -> Any: ...
def HasStringPropWithValueQueryBond(*args, **kwargs) -> Any: ...
def HybridizationEqualsQueryAtom(*args, **kwargs) -> Any: ...
def HybridizationGreaterQueryAtom(*args, **kwargs) -> Any: ...
def HybridizationLessQueryAtom(*args, **kwargs) -> Any: ...
def InNRingsEqualsQueryAtom(*args, **kwargs) -> Any: ...
def InNRingsGreaterQueryAtom(*args, **kwargs) -> Any: ...
def InNRingsLessQueryAtom(*args, **kwargs) -> Any: ...
def IsAliphaticQueryAtom(*args, **kwargs) -> Any: ...
def IsAromaticQueryAtom(*args, **kwargs) -> Any: ...
def IsInRingQueryAtom(*args, **kwargs) -> Any: ...
def IsUnsaturatedQueryAtom(*args, **kwargs) -> Any: ...
def IsotopeEqualsQueryAtom(*args, **kwargs) -> Any: ...
def IsotopeGreaterQueryAtom(*args, **kwargs) -> Any: ...
def IsotopeLessQueryAtom(*args, **kwargs) -> Any: ...
def MAtomQueryAtom(*args, **kwargs) -> Any: ...
def MHAtomQueryAtom(*args, **kwargs) -> Any: ...
def MassEqualsQueryAtom(*args, **kwargs) -> Any: ...
def MassGreaterQueryAtom(*args, **kwargs) -> Any: ...
def MassLessQueryAtom(*args, **kwargs) -> Any: ...
def MinRingSizeEqualsQueryAtom(*args, **kwargs) -> Any: ...
def MinRingSizeGreaterQueryAtom(*args, **kwargs) -> Any: ...
def MinRingSizeLessQueryAtom(*args, **kwargs) -> Any: ...
def MissingChiralTagQueryAtom(*args, **kwargs) -> Any: ...
def NumAliphaticHeteroatomNeighborsEqualsQueryAtom(*args, **kwargs) -> Any: ...
def NumAliphaticHeteroatomNeighborsGreaterQueryAtom(*args, **kwargs) -> Any: ...
def NumAliphaticHeteroatomNeighborsLessQueryAtom(*args, **kwargs) -> Any: ...
def NumHeteroatomNeighborsEqualsQueryAtom(*args, **kwargs) -> Any: ...
def NumHeteroatomNeighborsGreaterQueryAtom(*args, **kwargs) -> Any: ...
def NumHeteroatomNeighborsLessQueryAtom(*args, **kwargs) -> Any: ...
def NumRadicalElectronsEqualsQueryAtom(*args, **kwargs) -> Any: ...
def NumRadicalElectronsGreaterQueryAtom(*args, **kwargs) -> Any: ...
def NumRadicalElectronsLessQueryAtom(*args, **kwargs) -> Any: ...
def QAtomQueryAtom(*args, **kwargs) -> Any: ...
def QHAtomQueryAtom(*args, **kwargs) -> Any: ...
def RingBondCountEqualsQueryAtom(*args, **kwargs) -> Any: ...
def RingBondCountGreaterQueryAtom(*args, **kwargs) -> Any: ...
def RingBondCountLessQueryAtom(*args, **kwargs) -> Any: ...
def TotalDegreeEqualsQueryAtom(*args, **kwargs) -> Any: ...
def TotalDegreeGreaterQueryAtom(*args, **kwargs) -> Any: ...
def TotalDegreeLessQueryAtom(*args, **kwargs) -> Any: ...
def TotalValenceEqualsQueryAtom(*args, **kwargs) -> Any: ...
def TotalValenceGreaterQueryAtom(*args, **kwargs) -> Any: ...
def TotalValenceLessQueryAtom(*args, **kwargs) -> Any: ...
def XAtomQueryAtom(*args, **kwargs) -> Any: ...
def XHAtomQueryAtom(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit.Chem.Draw.canvasbase import CanvasBase as CanvasBase

class Canvas(CanvasBase):
    _name: Incomplete
    size: Incomplete
    _figure: Incomplete
    _axes: Incomplete
    _dpi: Incomplete
    def __init__(self, size, name: str = ..., imageType: str = ...) -> None: ...
    def rescalePt(self, p1): ...
    def addCanvasLine(
        self, p1, p2, color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def addCanvasText(self, text, pos, font, color=..., **kwargs): ...
    def addCanvasPolygon(self, ps, color=..., **kwargs) -> None: ...
    def addCanvasDashedWedge(
        self, p1, p2, p3, dash=..., color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...



from _typeshed import Incomplete

class DescriptorCalculator:
    simpleList: Incomplete
    descriptorNames: Incomplete
    compoundList: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def ShowDescriptors(self) -> None: ...
    def GetDescriptorNames(self) -> None: ...
    def SaveState(self, fileName) -> None: ...
    def CalcDescriptors(self, what, *args, **kwargs) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit import Chem as Chem, RDConfig as RDConfig
from rdkit.Chem.rdmolfiles import (
    SDMolSupplier as SDMolSupplier,
    SmilesMolSupplier as SmilesMolSupplier,
)

class InputFormat:
    SMARTS: str
    MOL: str
    SMILES: str

def _smartsFromSmartsLine(line): ...
def _getSmartsSaltsFromStream(stream) -> Generator[Incomplete, None, None]: ...
def _getSmartsSaltsFromFile(filename): ...

class SaltRemover:
    defnFilename: Incomplete
    defnData: Incomplete
    salts: Incomplete
    defnFormat: Incomplete
    def __init__(
        self,
        defnFilename: Incomplete | None = ...,
        defnData: Incomplete | None = ...,
        defnFormat=...,
    ) -> None: ...
    def _initPatterns(self) -> None: ...
    def StripMol(self, mol, dontRemoveEverything: bool = ..., sanitize: bool = ...): ...
    def StripMolWithDeleted(self, mol, dontRemoveEverything: bool = ...): ...
    def _StripMol(
        self, mol, dontRemoveEverything: bool = ..., sanitize: bool = ...
    ): ...
    def __call__(self, mol, dontRemoveEverything: bool = ...): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from numpy import *
from _typeshed import Incomplete
from rdkit.sping import pid as pid

def DrawSpiral(
    canvas,
    startColor,
    endColor,
    startRadius,
    endRadius,
    nLoops,
    degsPerSlice: int = ...,
    degsPerStep: int = ...,
    startAngle: int = ...,
    centerPos: Incomplete | None = ...,
    dir: int = ...,
) -> None: ...



from typing import Any, ClassVar

from typing import overload

class ContourParams:
    __instance_size__: ClassVar[int] = ...
    contourWidth: Any
    dashNegative: Any
    extraGridPadding: Any
    fillGrid: Any
    gridResolution: Any
    setScale: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def setColourMap(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setContourColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class IntStringMap:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class MolDraw2D:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def ClearDrawing(cls, RDKit) -> Any: ...
    @classmethod
    def DrawArc(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawArrow(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawAttachmentLine(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawEllipse(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawLine(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawMolecule(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawMoleculeWithHighlights(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawMolecules(cls, RDKit, boost) -> Any: ...
    @classmethod
    def DrawPolygon(cls, RDKit, boost) -> Any: ...
    @classmethod
    def DrawReaction(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawRect(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawString(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawTriangle(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def DrawWavyLine(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def FillPolys(cls, RDKit) -> Any: ...
    @classmethod
    def FontSize(cls, RDKit) -> Any: ...
    @overload
    @classmethod
    def GetDrawCoords(cls, RDKit, RDGeom) -> Any: ...
    @overload
    @classmethod
    def GetDrawCoords(cls, RDKit, int) -> Any: ...
    @classmethod
    def Height(cls, RDKit) -> Any: ...
    @classmethod
    def LineWidth(cls, RDKit) -> Any: ...
    @classmethod
    def Offset(cls, RDKit) -> Any: ...
    @classmethod
    def SetColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def SetDrawOptions(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetFillPolys(cls, RDKit, bool) -> Any: ...
    @classmethod
    def SetFontSize(cls, RDKit, double) -> Any: ...
    @classmethod
    def SetLineWidth(cls, RDKit, int) -> Any: ...
    @classmethod
    def SetOffset(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetScale(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Width(cls, RDKit) -> Any: ...
    @classmethod
    def drawOptions(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolDraw2DCairo(MolDraw2D):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def FinishDrawing(cls, RDKit) -> Any: ...
    @classmethod
    def GetDrawingText(cls, RDKit) -> Any: ...
    @classmethod
    def WriteDrawingText(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolDraw2DSVG(MolDraw2D):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddMoleculeMetadata(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def FinishDrawing(cls, RDKit) -> Any: ...
    @classmethod
    def GetDrawingText(cls, RDKit) -> Any: ...
    @classmethod
    def TagAtoms(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolDrawOptions:
    __instance_size__: ClassVar[int] = ...
    addAtomIndices: Any
    addBondIndices: Any
    addStereoAnnotation: Any
    additionalAtomLabelPadding: Any
    annotationFontScale: Any
    atomHighlightsAreCircles: Any
    atomLabelDeuteriumTritium: Any
    atomLabels: Any
    atomRegions: Any
    baseFontSize: Any
    bondLineWidth: Any
    centreMoleculesBeforeDrawing: Any
    circleAtoms: Any
    clearBackground: Any
    comicMode: Any
    continuousHighlight: Any
    drawMolsSameScale: Any
    dummiesAreAttachments: Any
    dummyIsotopeLabels: Any
    explicitMethyl: Any
    fillHighlights: Any
    fixedBondLength: Any
    fixedFontSize: Any
    fixedScale: Any
    flagCloseContactsDist: Any
    fontFile: Any
    highlightBondWidthMultiplier: Any
    highlightRadius: Any
    includeAtomTags: Any
    includeChiralFlagLabel: Any
    includeMetadata: Any
    includeRadicals: Any
    isotopeLabels: Any
    legendFontSize: Any
    legendFraction: Any
    maxFontSize: Any
    minFontSize: Any
    multipleBondOffset: Any
    noAtomLabels: Any
    padding: Any
    prepareMolsBeforeDrawing: Any
    rotate: Any
    scaleBondWidth: Any
    scaleHighlightBondWidth: Any
    scalingFactor: Any
    simplifiedStereoGroupLabel: Any
    singleColourWedgeBonds: Any
    splitBonds: Any
    variableAtomRadius: Any
    variableBondWidthMultiplier: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def getAnnotationColour(cls, RDKit) -> Any: ...
    @classmethod
    def getBackgroundColour(cls, RDKit) -> Any: ...
    @classmethod
    def getHighlightColour(cls, RDKit) -> Any: ...
    @classmethod
    def getLegendColour(cls, RDKit) -> Any: ...
    @classmethod
    def getSymbolColour(cls, RDKit) -> Any: ...
    @classmethod
    def getVariableAttachmentColour(cls, RDKit) -> Any: ...
    @classmethod
    def setAnnotationColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setAtomPalette(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setBackgroundColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setHighlightColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setLegendColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setSymbolColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def setVariableAttachmentColour(cls, RDKit, boost) -> Any: ...
    @classmethod
    def updateAtomPalette(cls, RDKit, boost) -> Any: ...
    @classmethod
    def useAvalonAtomPalette(cls, RDKit) -> Any: ...
    @classmethod
    def useBWAtomPalette(cls, RDKit) -> Any: ...
    @classmethod
    def useCDKAtomPalette(cls, RDKit) -> Any: ...
    @classmethod
    def useDefaultAtomPalette(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class map_indexing_suite_IntStringMap_entry:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def data(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def key(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def ContourAndDrawGaussians(*args, **kwargs) -> Any: ...
def ContourAndDrawGrid(*args, **kwargs) -> Any: ...
def MolToSVG(RDKit) -> Any: ...
def PrepareAndDrawMolecule(*args, **kwargs) -> Any: ...
def PrepareMolForDrawing(RDKit) -> Any: ...
@overload
def SetDarkMode(arg1: MolDrawOptions) -> Any: ...
@overload
def SetDarkMode(arg1: MolDraw2D) -> Any: ...
def SetMonochromeMode(*args, **kwargs) -> Any: ...
def UpdateDrawerParamsFromJSON(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit.Dbase import DbInfo as DbInfo, DbModule as DbModule
from rdkit.Dbase.DbResultSet import (
    DbResultSet as DbResultSet,
    RandomAccessDbResultSet as RandomAccessDbResultSet,
)

def _take(fromL, what): ...
def GetColumns(
    dBase,
    table,
    fieldString,
    user: str = ...,
    password: str = ...,
    join: str = ...,
    cn: Incomplete | None = ...,
): ...
def GetData(
    dBase,
    table,
    fieldString: str = ...,
    whereString: str = ...,
    user: str = ...,
    password: str = ...,
    removeDups: int = ...,
    join: str = ...,
    forceList: int = ...,
    transform: Incomplete | None = ...,
    randomAccess: int = ...,
    extras: Incomplete | None = ...,
    cn: Incomplete | None = ...,
): ...
def DatabaseToText(
    dBase,
    table,
    fields: str = ...,
    join: str = ...,
    where: str = ...,
    user: str = ...,
    password: str = ...,
    delim: str = ...,
    cn: Incomplete | None = ...,
): ...
def TypeFinder(data, nRows, nCols, nullMarker: Incomplete | None = ...): ...
def _AdjustColHeadings(colHeadings, maxColLabelLen): ...
def GetTypeStrings(colHeadings, colTypes, keyCol: Incomplete | None = ...): ...
def _insertBlock(conn, sqlStr, block, silent: bool = ...): ...
def _AddDataToDb(
    dBase,
    table,
    user,
    password,
    colDefs,
    colTypes,
    data,
    nullMarker: Incomplete | None = ...,
    blockSize: int = ...,
    cn: Incomplete | None = ...,
) -> None: ...
def TextFileToDatabase(
    dBase,
    table,
    inF,
    delim: str = ...,
    user: str = ...,
    password: str = ...,
    maxColLabelLen: int = ...,
    keyCol: Incomplete | None = ...,
    nullMarker: Incomplete | None = ...,
) -> None: ...
def DatabaseToDatabase(
    fromDb,
    fromTbl,
    toDb,
    toTbl,
    fields: str = ...,
    join: str = ...,
    where: str = ...,
    user: str = ...,
    password: str = ...,
    keyCol: Incomplete | None = ...,
    nullMarker: str = ...,
) -> None: ...



from rdkit.sping.pid import *
from math import *
from _typeshed import Incomplete
from rdkit.sping.PDF import pdfmetrics as pdfmetrics

SVG_HEADER: str

def _ColorToSVG(color): ...
def _PointListToSVG(points, dupFirst: int = ...): ...

class SVGCanvas(Canvas):
    _nImages: int
    _imageFormat: str
    size: Incomplete
    def __init__(
        self,
        size=...,
        name: str = ...,
        includeXMLHeader: bool = ...,
        extraHeaderText: str = ...,
    ) -> None: ...
    _txt: Incomplete
    def _initOutput(
        self, includeXMLHeader: bool = ..., extraHeaderText: str = ...
    ) -> None: ...
    def _findExternalFontName(self, font): ...
    def _FormFontStr(self, font): ...
    def _FormArcStr(self, x1, y1, x2, y2, theta1, extent): ...
    def clear(self) -> None: ...
    def flush(self) -> None: ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def text(self): ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor=...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawEllipse(
        self,
        x1,
        y1,
        x2,
        y2,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor=...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def _drawStringOneLine(self, line, x, y, fontStr, svgColor, **kwargs): ...
    def drawFigure(
        self,
        partList,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawImage(
        self,
        image,
        x1,
        y1,
        x2: Incomplete | None = ...,
        y2: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def stringWidth(self, s, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...): ...
    def fontDescent(self, font: Incomplete | None = ...): ...

def test(): ...
def dashtest(): ...
def testit(canvas, s, x, y, font: Incomplete | None = ...) -> None: ...
def test2() -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit import Chem as Chem
from rdkit.Chem import AllChem as AllChem

def GetHeterocycleReactionSmarts(): ...
def _ParseReactions() -> Generator[Incomplete, None, None]: ...

REACTION_CACHE: Incomplete

def GetHeterocycleReactions(): ...
def EnumerateHeterocycles(
    inputmol, depth: Incomplete | None = ...
) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, Geometry as Geometry
from rdkit.Chem import AllChem as AllChem
from rdkit.Chem.PyMol import MolViewer as MolViewer
from rdkit.Chem.Subshape import (
    SubshapeAligner as SubshapeAligner,
    SubshapeBuilder as SubshapeBuilder,
    SubshapeObjects as SubshapeObjects,
)

m1: Incomplete
m2: Incomplete
b: Incomplete
s1: Incomplete
s2: Incomplete
ns1: Incomplete
v: Incomplete
a: Incomplete
pruneStats: Incomplete
algs: Incomplete
fName: Incomplete



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import (
    Graphs as Graphs,
    rdMolDescriptors as rdMolDescriptors,
    rdchem as rdchem,
)
from rdkit.ML.InfoTheory import entropy as entropy

ptable: Incomplete
_log2val: Incomplete

def _VertexDegrees(mat, onlyOnes: int = ...): ...
def _NumAdjacencies(mol, dMat): ...
def _GetCountDict(arr): ...

hallKierAlphas: Incomplete

def _pyHallKierAlpha(m): ...
def Ipc(mol, avg: int = ..., dMat: Incomplete | None = ..., forceDMat: int = ...): ...
def _pyKappa1(mol): ...
def _pyKappa2(mol): ...
def _pyKappa3(mol): ...

HallKierAlpha: Incomplete
Kappa1: Incomplete
Kappa2: Incomplete
Kappa3: Incomplete

def Chi0(mol): ...
def Chi1(mol): ...
def _nVal(atom): ...
def _hkDeltas(mol, skipHs: int = ...): ...
def _pyChi0v(mol): ...
def _pyChi1v(mol): ...
def _pyChiNv_(mol, order: int = ...): ...
def _pyChi2v(mol): ...
def _pyChi3v(mol): ...
def _pyChi4v(mol): ...
def _pyChi0n(mol): ...
def _pyChi1n(mol): ...
def _pyChiNn_(mol, order: int = ...): ...
def _pyChi2n(mol): ...
def _pyChi3n(mol): ...
def _pyChi4n(mol): ...

Chi0v: Incomplete
Chi1v: Incomplete
Chi2v: Incomplete
Chi3v: Incomplete
Chi4v: Incomplete
ChiNv_: Incomplete
Chi0n: Incomplete
Chi1n: Incomplete
Chi2n: Incomplete
Chi3n: Incomplete
Chi4n: Incomplete
ChiNn_: Incomplete

def BalabanJ(mol, dMat: Incomplete | None = ..., forceDMat: int = ...): ...
def _AssignSymmetryClasses(mol, vdList, bdMat, forceBDMat, numAtoms, cutoff): ...
def _LookUpBondOrder(atom1Id, atom2Id, bondDic): ...
def _CalculateEntropies(connectionDict, atomTypeDict, numAtoms): ...
def _CreateBondDictEtc(mol, numAtoms): ...
def BertzCT(
    mol, cutoff: int = ..., dMat: Incomplete | None = ..., forceDMat: int = ...
): ...



from typing import Any, ClassVar

class FreeChemicalFeature:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetFamily(cls, ChemicalFeatures) -> Any: ...
    @classmethod
    def GetId(cls, ChemicalFeatures) -> Any: ...
    @classmethod
    def GetPos(cls, ChemicalFeatures) -> Any: ...
    @classmethod
    def GetType(cls, ChemicalFeatures) -> Any: ...
    @classmethod
    def SetFamily(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetId(cls, ChemicalFeatures, int) -> Any: ...
    @classmethod
    def SetPos(cls, ChemicalFeatures, RDGeom) -> Any: ...
    @classmethod
    def SetType(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __getinitargs__(cls, ChemicalFeatures) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...



from typing import Any, ClassVar

AtomPairFP: FPType
MorganFP: FPType
RDKitFP: FPType
TopologicalTorsionFP: FPType

class AdditionalOutput:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AllocateAtomCounts(cls, RDKit) -> Any: ...
    @classmethod
    def AllocateAtomToBits(cls, RDKit) -> Any: ...
    @classmethod
    def AllocateBitInfoMap(cls, RDKit) -> Any: ...
    @classmethod
    def AllocateBitPaths(cls, RDKit) -> Any: ...
    @classmethod
    def GetAtomCounts(cls, RDKit) -> Any: ...
    @classmethod
    def GetAtomToBits(cls, RDKit) -> Any: ...
    @classmethod
    def GetBitInfoMap(cls, RDKit) -> Any: ...
    @classmethod
    def GetBitPaths(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class AtomInvariantsGenerator:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class BondInvariantsGenerator:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FPType:
    AtomPairFP: ClassVar[FPType] = ...
    MorganFP: ClassVar[FPType] = ...
    RDKitFP: ClassVar[FPType] = ...
    TopologicalTorsionFP: ClassVar[FPType] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class FingerprintGenerator32:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetCountFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetCountFingerprintAsNumPy(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFingerprintAsNumPy(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetInfoString(cls, RDKit) -> Any: ...
    @classmethod
    def GetSparseCountFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetSparseFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FingerprintGenerator64:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetCountFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetCountFingerprintAsNumPy(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFingerprintAsNumPy(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetInfoString(cls, RDKit) -> Any: ...
    @classmethod
    def GetSparseCountFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetSparseFingerprint(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def GetAtomPairAtomInvGen(*args, **kwargs) -> Any: ...
def GetAtomPairGenerator(*args, **kwargs) -> Any: ...
def GetCountFPs(*args, **kwargs) -> Any: ...
def GetFPs(*args, **kwargs) -> Any: ...
def GetMorganAtomInvGen(*args, **kwargs) -> Any: ...
def GetMorganBondInvGen(*args, **kwargs) -> Any: ...
def GetMorganFeatureAtomInvGen(*args, **kwargs) -> Any: ...
def GetMorganGenerator(*args, **kwargs) -> Any: ...
def GetRDKitAtomInvGen() -> Any: ...
def GetRDKitFPGenerator(*args, **kwargs) -> Any: ...
def GetSparseCountFPs(*args, **kwargs) -> Any: ...
def GetSparseFPs(*args, **kwargs) -> Any: ...
def GetTopologicalTorsionGenerator(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit.ML.Data import SplitData as SplitData
from rdkit.ML.NaiveBayes.ClassificationModel import (
    NaiveBayesClassifier as NaiveBayesClassifier,
)

def makeNBClassificationModel(
    trainExamples,
    attrs,
    nPossibleValues,
    nQuantBounds,
    mEstimateVal=...,
    useSigs: bool = ...,
    ensemble: Incomplete | None = ...,
    useCMIM: int = ...,
    **kwargs
): ...
def CrossValidate(NBmodel, testExamples, appendExamples: int = ...): ...
def CrossValidationDriver(
    examples,
    attrs,
    nPossibleValues,
    nQuantBounds,
    mEstimateVal: float = ...,
    holdOutFrac: float = ...,
    modelBuilder=...,
    silent: int = ...,
    calcTotalError: int = ...,
    **kwargs
): ...



from rdkit import Chem as Chem, Geometry as Geometry

def cross(v1, v2): ...
def findNeighbors(atomId, adjMat): ...
def _findAvgVec(conf, center, nbrs): ...
def GetAromaticFeatVects(conf, featAtoms, featLoc, scale: float = ...): ...
def ArbAxisRotation(theta, ax, pt): ...
def GetAcceptor2FeatVects(conf, featAtoms, scale: float = ...): ...
def _GetTetrahedralFeatVect(conf, aid, scale): ...
def GetDonor3FeatVects(conf, featAtoms, scale: float = ...): ...
def GetAcceptor3FeatVects(conf, featAtoms, scale: float = ...): ...
def _findHydAtoms(nbrs, atomNames): ...
def _checkPlanarity(conf, cpt, nbrs, tol: float = ...): ...
def GetDonor2FeatVects(conf, featAtoms, scale: float = ...): ...
def GetDonor1FeatVects(conf, featAtoms, scale: float = ...): ...
def GetAcceptor1FeatVects(conf, featAtoms, scale: float = ...): ...



from _typeshed import Incomplete
from rdkit.ML.Descriptors import Descriptors as Descriptors
from rdkit.RDLogger import logger as logger

class MolecularDescriptorCalculator(Descriptors.DescriptorCalculator):
    simpleList: Incomplete
    descriptorNames: Incomplete
    compoundList: Incomplete
    def __init__(self, simpleList, *args, **kwargs) -> None: ...
    descriptorVersions: Incomplete
    def _findVersions(self) -> None: ...
    def SaveState(self, fileName) -> None: ...
    def CalcDescriptors(self, mol, *args, **kwargs): ...
    def GetDescriptorNames(self): ...
    def GetDescriptorSummaries(self): ...
    def GetDescriptorFuncs(self): ...
    def GetDescriptorVersions(self): ...



from _typeshed import Incomplete

StandardEnglishFonts: Incomplete
widths: Incomplete
ascent_descent: Incomplete

def parseAFMfile(filename): ...

class FontCache:
    __widtharrays: Incomplete
    def __init__(self) -> None: ...
    def loadfont(self, fontname) -> None: ...
    def getfont(self, fontname): ...
    def stringwidth(self, text, font): ...
    def status(self): ...

TheFontCache: Incomplete
stringwidth: Incomplete



from _typeshed import Incomplete

def _CenterTForm(dataSet) -> None: ...
def _NormalizeTForm(dataSet) -> None: ...
def _StandardTForm(dataSet) -> None: ...

_availTransforms: Incomplete

def GetAvailTransforms(): ...



from _typeshed import Incomplete
from rdkit.ML.Data import Quantize as Quantize

def _getBinId(val, qBounds): ...

class NaiveBayesClassifier:
    _attrs: Incomplete
    _mEstimateVal: Incomplete
    _useSigs: Incomplete
    _classProbs: Incomplete
    _examples: Incomplete
    _trainingExamples: Incomplete
    _testExamples: Incomplete
    _badExamples: Incomplete
    _QBoundVals: Incomplete
    _nClasses: Incomplete
    _qBounds: Incomplete
    _nPosVals: Incomplete
    _needsQuant: int
    _name: str
    mprob: Incomplete
    _condProbs: Incomplete
    def __init__(
        self, attrs, nPossibleVals, nQuantBounds, mEstimateVal=..., useSigs: bool = ...
    ) -> None: ...
    def GetName(self): ...
    def SetName(self, name) -> None: ...
    def NameModel(self, varNames) -> None: ...
    def GetExamples(self): ...
    def SetExamples(self, examples) -> None: ...
    def GetTrainingExamples(self): ...
    def SetTrainingExamples(self, examples) -> None: ...
    def GetTestExamples(self): ...
    def SetTestExamples(self, examples) -> None: ...
    def SetBadExamples(self, examples) -> None: ...
    def GetBadExamples(self): ...
    def _computeQuantBounds(self) -> None: ...
    def trainModel(self) -> None: ...
    def ClassifyExamples(self, examples, appendExamples: int = ...): ...
    def GetClassificationDetails(self): ...
    def ClassifyExample(self, example, appendExamples: int = ...): ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs
from rdkit.Dbase import DbModule as DbModule
from rdkit.Dbase.DbConnection import DbConnect as DbConnect
from rdkit.ML import CompositeRun as CompositeRun
from rdkit.ML.Data import DataUtils as DataUtils, SplitData as SplitData

hasPil: int
_details: Incomplete
__VERSION_STRING: str

def message(msg, noRet: int = ...) -> None: ...
def error(msg) -> None: ...
def CalcEnrichment(mat, tgt: int = ...): ...
def CollectResults(
    indices,
    dataSet,
    composite,
    callback: Incomplete | None = ...,
    appendExamples: int = ...,
    errorEstimate: int = ...,
): ...
def DetailedScreen(
    indices,
    data,
    composite,
    threshold: int = ...,
    screenResults: Incomplete | None = ...,
    goodVotes: Incomplete | None = ...,
    badVotes: Incomplete | None = ...,
    noVotes: Incomplete | None = ...,
    callback: Incomplete | None = ...,
    appendExamples: int = ...,
    errorEstimate: int = ...,
) -> None: ...
def ShowVoteResults(
    indices,
    data,
    composite,
    nResultCodes,
    threshold,
    verbose: int = ...,
    screenResults: Incomplete | None = ...,
    callback: Incomplete | None = ...,
    appendExamples: int = ...,
    goodVotes: Incomplete | None = ...,
    badVotes: Incomplete | None = ...,
    noVotes: Incomplete | None = ...,
    errorEstimate: int = ...,
): ...
def ScreenIt(
    composite,
    indices,
    data,
    partialVote: int = ...,
    voteTol: float = ...,
    verbose: int = ...,
    screenResults: Incomplete | None = ...,
    goodVotes: Incomplete | None = ...,
    badVotes: Incomplete | None = ...,
    noVotes: Incomplete | None = ...,
): ...
def _processVoteList(votes, data) -> None: ...
def PrepareDataFromDetails(model, details, data, verbose: int = ...): ...
def ScreenFromDetails(
    models,
    details,
    callback: Incomplete | None = ...,
    setup: Incomplete | None = ...,
    appendExamples: int = ...,
    goodVotes: Incomplete | None = ...,
    badVotes: Incomplete | None = ...,
    noVotes: Incomplete | None = ...,
    data: Incomplete | None = ...,
    enrichments: Incomplete | None = ...,
): ...
def GetScreenImage(nGood, nBad, nRej, size: Incomplete | None = ...): ...
def ScreenToHtml(
    nGood,
    nBad,
    nRej,
    avgGood,
    avgBad,
    avgSkip,
    voteTable,
    imgDir: str = ...,
    fullPage: int = ...,
    skipImg: int = ...,
    includeDefs: int = ...,
): ...
def MakePredPlot(
    details,
    indices,
    data,
    goodVotes,
    badVotes,
    nRes,
    idCol: int = ...,
    verbose: int = ...,
) -> None: ...
def Go(details) -> None: ...
def SetDefaults(details: Incomplete | None = ...): ...
def Usage() -> None: ...
def ShowVersion(includeArgs: int = ...) -> None: ...
def ParseArgs(details): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

_nextDisplayId: int

class Displayable:
    doc: Incomplete
    id: Incomplete
    visible: bool
    children: Incomplete
    def __init__(self, doc, id: int = ...) -> None: ...
    def Select(self, atoms=..., state: bool = ..., recurse: bool = ...): ...
    def Hide(self, recurse: bool = ...) -> None: ...
    def Show(self, recurse: bool = ...) -> None: ...
    def ShowOnly(self, recurse: bool = ...) -> None: ...
    def __del__(self) -> None: ...

class MolViewer:
    app: Incomplete
    doc: Incomplete
    displayables: Incomplete
    def __init__(self, force: int = ..., title: str = ..., **kwargs) -> None: ...
    def DeleteAll(self) -> None: ...
    def DeleteAllExcept(self, excludes) -> None: ...
    def ShowMol(
        self,
        mol,
        name: str = ...,
        showOnly: bool = ...,
        highlightFeatures=...,
        molB: str = ...,
        confId: int = ...,
        zoom: bool = ...,
    ) -> None: ...
    def LoadFile(self, filename, name, showOnly: bool = ...): ...
    def GetSelectedAtoms(self, whichSelection: str = ...): ...
    def HighlightAtoms(self, indices, where, extraHighlight: bool = ...) -> None: ...
    def SelectAtoms(self, itemId, atomIndices, selName: str = ...) -> None: ...
    def SetDisplayUpdate(self, val) -> None: ...
    def GetAtomCoords(self, sels): ...
    def AddPharmacophore(self, locs, colors, label, sphereRad: float = ...) -> None: ...
    def SetDisplayStyle(self, obj, style: str = ...) -> None: ...
    def HideAll(self) -> None: ...
    def HideObject(self, objName) -> None: ...
    def DisplayObject(self, objName) -> None: ...
    def Zoom(self, objName) -> None: ...
    def SelectProteinNeighborhood(
        self,
        aroundObj,
        inObj,
        distance: float = ...,
        name: str = ...,
        showSurface: bool = ...,
    ) -> None: ...
    def Redraw(self) -> None: ...



from rdkit.Dbase.DbConnection import DbConnect as DbConnect
from rdkit.ML import ScreenComposite as ScreenComposite
from rdkit.ML.Data import Stats as Stats
from rdkit.ML.DecTree import Tree as Tree, TreeUtils as TreeUtils

__VERSION_STRING: str

def ProcessIt(composites, nToConsider: int = ..., verbose: int = ...): ...
def ErrorStats(conn, where, enrich: int = ...): ...
def ShowStats(statD, enrich: int = ...) -> None: ...
def Usage() -> None: ...



from _typeshed import Incomplete
from rdkit import RDLogger as RDLogger

logger: Incomplete

def EuclideanDist(pi, pj): ...
def ClusterData(
    data, nPts, distThresh, isDistData: bool = ..., distFunc=..., reordering: bool = ...
): ...



from _typeshed import Incomplete

def EuclideanDistance(inData): ...
def CalcMetricMatrix(inData, metricFunc): ...
def FindMinValInList(mat, nObjs, minIdx: Incomplete | None = ...): ...
def ShowMetricMat(metricMat, nObjs) -> None: ...

methods: Incomplete



from rdkit.Chem.rdfiltercatalog import *
from _typeshed import Incomplete
from rdkit import Chem as Chem

class FilterMatcher(PythonFilterMatcher):
    name: Incomplete
    def __init__(self, name: str = ...) -> None: ...
    def GetMatches(self, mol, matchVect): ...
    def GetName(self): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, DataStructs as DataStructs
from rdkit.Chem import rdqueries as rdqueries

rdkitFpParams: Incomplete
FTYPE_ACYCLIC: str
FTYPE_CYCLIC: str
FTYPE_CYCLIC_ACYCLIC: str
ACYC_SMARTS: Incomplete
CYC_SMARTS: Incomplete
cSma1: Incomplete
cSma2: Incomplete
dummyAtomQuery: Incomplete

def delete_bonds(mol, bonds, ftype, hac): ...
def select_fragments(fragments, ftype, hac): ...
def isValidRingCut(mol): ...
def generate_fraggle_fragmentation(mol, verbose: bool = ...): ...
def atomContrib(subs, mol, tverskyThresh: float = ...): ...

modified_query_fps: Incomplete

def compute_fraggle_similarity_for_subs(
    inMol, qMol, qSmi, qSubs, tverskyThresh: float = ...
): ...
def GetFraggleSimilarity(queryMol, refMol, tverskyThresh: float = ...): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from .utils import memoized_property as memoized_property
from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import AllChem as AllChem

log: Incomplete

class Normalization:
    name: Incomplete
    transform_str: Incomplete
    def __init__(self, name, transform) -> None: ...
    def transform(self): ...
    def __repr__(self): ...
    def __str__(self): ...

NORMALIZATIONS: Incomplete
MAX_RESTARTS: int

class Normalizer:
    normalizations: Incomplete
    max_restarts: Incomplete
    def __init__(self, normalizations=..., max_restarts=...) -> None: ...
    def __call__(self, mol): ...
    def normalize(self, mol): ...
    def _normalize_fragment(self, mol): ...
    def _apply_transform(self, mol, rule): ...



from . import rdMolDraw2D as rdMolDraw2D
from _typeshed import Incomplete
from rdkit import Chem as Chem

rdkitStructureRendererJsUrl: str
minimalLibJsUrl: str
parentNodeQuery: str
_enabled: bool
_camelCaseOptToTagRe: Incomplete
_opts: str
_disabled: str
_defaultDrawOptions: Incomplete
_defaultDrawOptionsDict: Incomplete

def _isAcceptedKeyValue(key, value): ...
def _getValueFromKey(d, key): ...
def _molDrawOptionsToDict(molDrawOptions=...): ...
def _isDrawOptionEqual(v1, v2): ...
def filterDefaultDrawOpts(molDrawOptions): ...
def setEnabled(): ...
def isEnabled(mol: Incomplete | None = ...): ...
def getOpts(mol): ...
def setOpts(mol, opts) -> None: ...
def setOpt(mol, key, value) -> None: ...
def clearOpts(mol) -> None: ...
def clearOpt(mol, key) -> None: ...
def setNoInteractive(mol, shouldSet: bool = ...) -> None: ...
def isNoInteractive(mol): ...
def injectHTMLHeaderBeforeTable(html): ...
def generateHTMLHeader(doc, element): ...
def newlineToXml(molblock): ...
def xmlToNewline(xmlblock): ...
def toDataMol(mol): ...
def _dashLower(m): ...
def camelCaseOptToDataTag(opt): ...
def generateHTMLBody(useSvg, mol, size, drawOptions: Incomplete | None = ...): ...



import logging
from .errors import StopValidateError as StopValidateError
from .validations import VALIDATIONS as VALIDATIONS
from _typeshed import Incomplete
from rdkit import Chem as Chem

SIMPLE_FORMAT: str
LONG_FORMAT: str

class LogHandler(logging.Handler):
    logs: Incomplete
    def __init__(self) -> None: ...
    @property
    def logmessages(self): ...
    def emit(self, record) -> None: ...
    def flush(self) -> None: ...
    def close(self) -> None: ...

class Validator:
    raw: Incomplete
    log: Incomplete
    handler: Incomplete
    validations: Incomplete
    def __init__(
        self,
        validations=...,
        log_format=...,
        level=...,
        stdout: bool = ...,
        raw: bool = ...,
    ) -> None: ...
    def __call__(self, mol): ...
    def validate(self, mol): ...

def validate_smiles(smiles): ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit import Chem as Chem
from typing import NamedTuple

__version__: str
__version_info: Incomplete

class Default:
    timeout: Incomplete
    timeoutString: str
    maximize: str
    atomCompare: str
    bondCompare: str
    matchValences: bool
    ringMatchesRingOnly: bool
    completeRingsOnly: bool

_get_symbol: Incomplete

class AtomSmartsNoAromaticity(dict):
    def __missing__(self, eleno): ...

_atom_smarts_no_aromaticity: Incomplete

def atom_typer_any(atoms): ...
def atom_typer_elements(atoms): ...
def atom_typer_isotopes(atoms): ...
def bond_typer_any(bonds): ...
def bond_typer_bondtypes(bonds): ...

atom_typers: Incomplete
bond_typers: Incomplete
default_atom_typer: Incomplete
default_bond_typer: Incomplete

def get_isotopes(mol): ...
def set_isotopes(mol, isotopes) -> None: ...

_isotope_dict: Incomplete
_atom_class_dict: Incomplete

def save_isotopes(mol, isotopes) -> None: ...
def save_atom_classes(mol, atom_classes) -> None: ...
def get_selected_atom_classes(mol, atom_indices): ...
def restore_isotopes(mol) -> None: ...
def assign_isotopes_from_class_tag(mol, atom_class_tag) -> None: ...

class TypedMolecule:
    rdmol: Incomplete
    rdmol_atoms: Incomplete
    rdmol_bonds: Incomplete
    atom_smarts_types: Incomplete
    bond_smarts_types: Incomplete
    canonical_bondtypes: Incomplete
    def __init__(
        self,
        rdmol,
        rdmol_atoms,
        rdmol_bonds,
        atom_smarts_types,
        bond_smarts_types,
        canonical_bondtypes,
    ) -> None: ...

class FragmentedTypedMolecule:
    rdmol: Incomplete
    rdmol_atoms: Incomplete
    orig_atoms: Incomplete
    orig_bonds: Incomplete
    atom_smarts_types: Incomplete
    bond_smarts_types: Incomplete
    canonical_bondtypes: Incomplete
    def __init__(
        self,
        rdmol,
        rdmol_atoms,
        orig_atoms,
        orig_bonds,
        atom_smarts_types,
        bond_smarts_types,
        canonical_bondtypes,
    ) -> None: ...

class TypedFragment:
    rdmol: Incomplete
    orig_atoms: Incomplete
    orig_bonds: Incomplete
    atom_smarts_types: Incomplete
    bond_smarts_types: Incomplete
    canonical_bondtypes: Incomplete
    def __init__(
        self,
        rdmol,
        orig_atoms,
        orig_bonds,
        atom_smarts_types,
        bond_smarts_types,
        canonical_bondtypes,
    ) -> None: ...

def get_canonical_bondtypes(rdmol, bonds, atom_smarts_types, bond_smarts_types): ...
def get_typed_molecule(
    rdmol, atom_typer, bond_typer, matchValences=..., ringMatchesRingOnly=...
): ...
def get_specified_types(rdmol, atom_types, ringMatchesRingOnly): ...
def convert_input_to_typed_molecules(
    mols, atom_typer, bond_typer, matchValences, ringMatchesRingOnly
): ...
def _check_atom_classes(molno, num_atoms, atom_classes) -> None: ...
def get_counts(it): ...
def intersect_counts(counts1, counts2): ...
def get_canonical_bondtype_counts(typed_mols): ...
def remove_unknown_bondtypes(typed_mol, supported_canonical_bondtypes): ...
def find_upper_fragment_size_limits(rdmol, atoms): ...

class EnumerationMolecule(NamedTuple):
    rdmol: Incomplete
    atoms: Incomplete
    bonds: Incomplete
    directed_edges: Incomplete

class Atom(NamedTuple):
    real_atom: Incomplete
    atom_smarts: Incomplete
    bond_indices: Incomplete
    is_in_ring: Incomplete

class Bond(NamedTuple):
    real_bond: Incomplete
    bond_smarts: Incomplete
    canonical_bondtype: Incomplete
    atom_indices: Incomplete
    is_in_ring: Incomplete

class DirectedEdge(NamedTuple):
    bond_index: Incomplete
    end_atom_index: Incomplete

class Subgraph(NamedTuple):
    atom_indices: Incomplete
    bond_indices: Incomplete

def get_typed_fragment(typed_mol, atom_indices): ...
def fragmented_mol_to_enumeration_mols(typed_mol, minNumAtoms: int = ...): ...
def gen_primes() -> Generator[Incomplete, None, None]: ...

_prime_stream: Incomplete
_primes: Incomplete

def _get_nth_prime(n): ...

class CangenNode:
    __slots__: Incomplete
    index: Incomplete
    atom_smarts: Incomplete
    value: int
    neighbors: Incomplete
    rank: int
    outgoing_edges: Incomplete
    def __init__(self, index, atom_smarts) -> None: ...

class OutgoingEdge(NamedTuple):
    from_atom_index: Incomplete
    bond_index: Incomplete
    bond_smarts: Incomplete
    other_node_idx: Incomplete
    other_node: Incomplete

def get_initial_cangen_nodes(
    subgraph, enumeration_mol, atom_assignment, do_initial_assignment: bool = ...
): ...
def rerank(cangen_nodes) -> None: ...
def find_duplicates(cangen_nodes, start, end): ...
def canon(cangen_nodes): ...
def get_closure_label(bond_smarts, closure): ...

_available_closures: Incomplete

def generate_smarts(cangen_nodes): ...
def make_canonical_smarts(subgraph, enumeration_mol, atom_assignment): ...
def make_arbitrary_smarts(subgraph, enumeration_mol, atom_assignment): ...
def powerset(iterable): ...
def nonempty_powerset(iterable): ...
def _Counter(): ...

tiebreaker: Incomplete

def find_extensions(atom_indices, visited_bond_indices, directed_edges): ...
def all_subgraph_extensions(
    enumeration_mol, subgraph, visited_bond_indices, internal_bonds, external_edges
) -> Generator[Incomplete, None, None]: ...
def find_extension_size(
    enumeration_mol, known_atoms, exclude_bonds, directed_edges
): ...

class CachingTargetsMatcher(dict):
    targets: Incomplete
    required_match_count: Incomplete
    _num_allowed_errors: Incomplete
    def __init__(
        self, targets, required_match_count: Incomplete | None = ...
    ) -> None: ...
    def shift_targets(self) -> None: ...
    def __missing__(self, smarts): ...

class VerboseCachingTargetsMatcher:
    targets: Incomplete
    cache: Incomplete
    required_match_count: Incomplete
    _num_allowed_errors: Incomplete
    num_lookups: int
    num_search_true: int
    def __init__(
        self, targets, required_match_count: Incomplete | None = ...
    ) -> None: ...
    def shift_targets(self) -> None: ...
    def __getitem__(self, smarts, missing=...): ...
    def report(self) -> None: ...

def prune_maximize_bonds(
    subgraph, mol, num_remaining_atoms, num_remaining_bonds, best_sizes
): ...
def prune_maximize_atoms(
    subgraph, mol, num_remaining_atoms, num_remaining_bonds, best_sizes
): ...

class _SingleBest:
    best_num_atoms: int
    best_smarts: Incomplete
    sizes: Incomplete
    timer: Incomplete
    verbose: Incomplete
    def __init__(self, timer, verbose) -> None: ...
    best_num_bonds: Incomplete
    def _new_best(self, num_atoms, num_bonds, smarts): ...
    def get_result(self, completed): ...

class MCSResult:
    num_atoms: Incomplete
    num_bonds: Incomplete
    smarts: Incomplete
    completed: Incomplete
    def __init__(self, num_atoms, num_bonds, smarts, completed) -> None: ...
    def __nonzero__(self): ...

class SingleBestAtoms(_SingleBest):
    def add_new_match(self, subgraph, mol, smarts): ...

class SingleBestBonds(_SingleBest):
    def add_new_match(self, subgraph, mol, smarts): ...

def check_completeRingsOnly(smarts, subgraph, enumeration_mol): ...

class SingleBestAtomsCompleteRingsOnly(_SingleBest):
    def add_new_match(self, subgraph, mol, smarts): ...

class SingleBestBondsCompleteRingsOnly(_SingleBest):
    def add_new_match(self, subgraph, mol, smarts): ...

_maximize_options: Incomplete

def enumerate_subgraphs(
    enumeration_mols,
    prune,
    atom_assignment,
    matches_all_targets,
    hits,
    timeout,
    heappush,
    heappop,
): ...

class Uniquer(dict):
    counter: Incomplete
    def __init__(self) -> None: ...
    def __missing__(self, key): ...

def MATCH(mol, pat): ...

class VerboseHeapOps:
    num_seeds_added: int
    num_seeds_processed: int
    verboseDelay: Incomplete
    _time_for_next_report: Incomplete
    trigger: Incomplete
    def __init__(self, trigger, verboseDelay) -> None: ...
    def heappush(self, seeds, item): ...
    def heappop(self, seeds): ...
    def trigger_report(self) -> None: ...
    def report(self) -> None: ...

def compute_mcs(
    fragmented_mols,
    typed_mols,
    minNumAtoms,
    threshold_count: Incomplete | None = ...,
    maximize=...,
    completeRingsOnly=...,
    timeout=...,
    timer: Incomplete | None = ...,
    verbose: bool = ...,
    verboseDelay: float = ...,
): ...

class Timer:
    mark_times: Incomplete
    def __init__(self) -> None: ...
    def mark(self, name) -> None: ...

def _update_times(timer, times) -> None: ...
def _get_threshold_count(num_mols, threshold): ...
def fmcs(
    mols,
    minNumAtoms: int = ...,
    maximize=...,
    atomCompare=...,
    bondCompare=...,
    threshold: float = ...,
    matchValences=...,
    ringMatchesRingOnly: bool = ...,
    completeRingsOnly: bool = ...,
    timeout=...,
    times: Incomplete | None = ...,
    verbose: bool = ...,
    verboseDelay: float = ...,
): ...
def subgraph_to_fragment(mol, subgraph): ...
def make_fragment_smiles(mcs, mol, subgraph, args: Incomplete | None = ...): ...
def _copy_sd_tags(mol, fragment) -> None: ...
def _MolToSDBlock(mol): ...
def _save_other_tags(mol, fragment, mcs, orig_mol, subgraph, args) -> None: ...
def make_fragment_sdf(mcs, mol, subgraph, args): ...
def make_complete_sdf(mcs, mol, subgraph, args): ...

structure_format_functions: Incomplete

def make_structure_format(format_name, mcs, mol, subgraph, args): ...
def parse_num_atoms(s): ...
def parse_threshold(s): ...
def parse_timeout(s): ...

class starting_from:
    left: Incomplete
    def __init__(self, left) -> None: ...
    def __contains__(self, value) -> bool: ...

range_pat: Incomplete
value_pat: Incomplete

def parse_select(s): ...

compare_shortcuts: Incomplete

def _get_match_bond_indices(pat, mol, match_atom_indices): ...
def main(args: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import (
    AllChem as AllChem,
    Crippen as Crippen,
    Descriptors as Descriptors,
    Lipinski as Lipinski,
)

decBase: Incomplete

class Compound(decBase):
    __tablename__: str
    guid: Incomplete
    molpkl: Incomplete

def RegisterSchema(dbUrl, echo: bool = ...): ...

ConnectToSchema = RegisterSchema

def _ConnectToSchema(dbUrl, echo: bool = ...): ...

logger: Incomplete

def ProcessMol(
    session,
    mol,
    globalProps,
    nDone,
    nameProp: str = ...,
    nameCol: str = ...,
    redraw: bool = ...,
    keepHs: bool = ...,
    skipProps: bool = ...,
    addComputedProps: bool = ...,
    skipSmiles: bool = ...,
): ...
def LoadDb(
    suppl,
    dbName,
    nameProp: str = ...,
    nameCol: str = ...,
    silent: bool = ...,
    redraw: bool = ...,
    errorsTo: Incomplete | None = ...,
    keepHs: bool = ...,
    defaultVal: str = ...,
    skipProps: bool = ...,
    regName: str = ...,
    skipSmiles: bool = ...,
    maxRowsCached: int = ...,
    uniqNames: bool = ...,
    addComputedProps: bool = ...,
    lazySupplier: bool = ...,
    numForPropScan: int = ...,
    startAnew: bool = ...,
) -> None: ...



from rdkit.Chem.Draw.rdMolDraw2D import *
from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig, rdBase as rdBase
from rdkit.Chem import rdDepictor as rdDepictor
from rdkit.Chem.Draw import rdMolDraw2D as rdMolDraw2D
from rdkit.Chem.Draw.MolDrawing import (
    DrawingOptions as DrawingOptions,
    MolDrawing as MolDrawing,
)
from typing import NamedTuple

def _getCanvas(): ...
def _createCanvas(size): ...
def _legacyMolToImage(
    mol, size, kekulize, wedgeBonds, fitImage, options, canvas, **kwargs
): ...
def _sip_available(): ...
def MolDraw2DFromQPainter(
    qpainter,
    width: int = ...,
    height: int = ...,
    panelWidth: int = ...,
    panelHeight: int = ...,
): ...
def MolToImage(
    mol,
    size=...,
    kekulize: bool = ...,
    wedgeBonds: bool = ...,
    fitImage: bool = ...,
    options: Incomplete | None = ...,
    canvas: Incomplete | None = ...,
    **kwargs
): ...
def _legacyMolToFile(
    mol, fileName, size, kekulize, wedgeBonds, imageType, fitImage, options, **kwargs
) -> None: ...
def MolToFile(
    mol,
    filename,
    size=...,
    kekulize: bool = ...,
    wedgeBonds: bool = ...,
    imageType: Incomplete | None = ...,
    fitImage: bool = ...,
    options: Incomplete | None = ...,
    **kwargs
) -> None: ...
def MolToImageFile(
    mol, filename, size=..., kekulize: bool = ..., wedgeBonds: bool = ..., **kwargs
) -> None: ...
def ShowMol(
    mol,
    size=...,
    kekulize: bool = ...,
    wedgeBonds: bool = ...,
    title: str = ...,
    stayInFront: bool = ...,
    **kwargs
) -> None: ...
def MolToMPL(
    mol,
    size=...,
    kekulize: bool = ...,
    wedgeBonds: bool = ...,
    imageType: Incomplete | None = ...,
    fitImage: bool = ...,
    options: Incomplete | None = ...,
    **kwargs
): ...
def _bivariate_normal(
    X,
    Y,
    sigmax: float = ...,
    sigmay: float = ...,
    mux: float = ...,
    muy: float = ...,
    sigmaxy: float = ...,
): ...
def calcAtomGaussians(
    mol, a: float = ..., step: float = ..., weights: Incomplete | None = ...
): ...
def MolsToImage(mols, subImgSize=..., legends: Incomplete | None = ..., **kwargs): ...
def _drawerToImage(d2d): ...
def _okToKekulizeMol(mol, kekulize): ...
def _moltoimg(
    mol,
    sz,
    highlights,
    legend,
    returnPNG: bool = ...,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def _moltoSVG(
    mol,
    sz,
    highlights,
    legend,
    kekulize,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def _MolsToGridImage(
    mols,
    molsPerRow: int = ...,
    subImgSize=...,
    legends: Incomplete | None = ...,
    highlightAtomLists: Incomplete | None = ...,
    highlightBondLists: Incomplete | None = ...,
    drawOptions: Incomplete | None = ...,
    returnPNG: bool = ...,
    **kwargs
): ...
def _MolsToGridSVG(
    mols,
    molsPerRow: int = ...,
    subImgSize=...,
    legends: Incomplete | None = ...,
    highlightAtomLists: Incomplete | None = ...,
    highlightBondLists: Incomplete | None = ...,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def MolsToGridImage(
    mols,
    molsPerRow: int = ...,
    subImgSize=...,
    legends: Incomplete | None = ...,
    highlightAtomLists: Incomplete | None = ...,
    highlightBondLists: Incomplete | None = ...,
    useSVG: bool = ...,
    returnPNG: bool = ...,
    **kwargs
): ...
def _legacyReactionToImage(rxn, subImgSize=..., **kwargs): ...
def ReactionToImage(
    rxn,
    subImgSize=...,
    useSVG: bool = ...,
    drawOptions: Incomplete | None = ...,
    returnPNG: bool = ...,
    **kwargs
): ...
def MolToQPixmap(
    mol,
    size=...,
    kekulize: bool = ...,
    wedgeBonds: bool = ...,
    fitImage: bool = ...,
    options: Incomplete | None = ...,
    **kwargs
): ...
def DrawMorganBit(mol, bitId, bitInfo, whichExample: int = ..., **kwargs): ...
def DrawMorganBits(tpls, **kwargs): ...

class FingerprintEnv(NamedTuple):
    submol: Incomplete
    highlightAtoms: Incomplete
    atomColors: Incomplete
    highlightBonds: Incomplete
    bondColors: Incomplete
    highlightRadii: Incomplete

def _getMorganEnv(
    mol,
    atomId,
    radius,
    baseRad,
    aromaticColor,
    ringColor,
    centerColor,
    extraColor,
    **kwargs
): ...
def DrawMorganEnvs(
    envs,
    molsPerRow: int = ...,
    subImgSize=...,
    baseRad: float = ...,
    useSVG: bool = ...,
    aromaticColor=...,
    ringColor=...,
    centerColor=...,
    extraColor=...,
    legends: Incomplete | None = ...,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def DrawMorganEnv(
    mol,
    atomId,
    radius,
    molSize=...,
    baseRad: float = ...,
    useSVG: bool = ...,
    aromaticColor=...,
    ringColor=...,
    centerColor=...,
    extraColor=...,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def DrawRDKitBits(tpls, **kwargs): ...
def DrawRDKitBit(mol, bitId, bitInfo, whichExample: int = ..., **kwargs): ...
def _getRDKitEnv(
    mol, bondPath, baseRad, aromaticColor, extraColor, nonAromaticColor, **kwargs
): ...
def DrawRDKitEnvs(
    envs,
    molsPerRow: int = ...,
    subImgSize=...,
    baseRad: float = ...,
    useSVG: bool = ...,
    aromaticColor=...,
    extraColor=...,
    nonAromaticColor: Incomplete | None = ...,
    legends: Incomplete | None = ...,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def DrawRDKitEnv(
    mol,
    bondPath,
    molSize=...,
    baseRad: float = ...,
    useSVG: bool = ...,
    aromaticColor=...,
    extraColor=...,
    nonAromaticColor: Incomplete | None = ...,
    drawOptions: Incomplete | None = ...,
    **kwargs
): ...
def SetComicMode(opts) -> None: ...



from rdkit.sping.colors import *
from _typeshed import Incomplete

__version_maj_number__: str
__version_min_number__: str
__version__: Incomplete
inch: int
cm: Incomplete

class StateSaver:
    canvas: Incomplete
    defaultLineColor: Incomplete
    defaultFillColor: Incomplete
    defaultLineWidth: Incomplete
    defaultFont: Incomplete
    def __init__(self, canvas) -> None: ...
    def __del__(self) -> None: ...

class Font:
    def __init__(
        self,
        size: int = ...,
        bold: int = ...,
        italic: int = ...,
        underline: int = ...,
        face: Incomplete | None = ...,
    ) -> None: ...
    def __cmp__(self, other): ...
    def __repr__(self): ...
    def __setattr__(self, name, value) -> None: ...

figureLine: int
figureArc: int
figureCurve: int
keyBksp: str
keyDel: str
keyLeft: str
keyRight: str
keyUp: str
keyDown: str
keyPgUp: str
keyPgDn: str
keyHome: str
keyEnd: str
keyClear: str
keyTab: str
modShift: int
modControl: int

class Canvas:
    defaultLineColor: Incomplete
    defaultFillColor: Incomplete
    defaultLineWidth: int
    defaultFont: Incomplete
    onClick: Incomplete
    onOver: Incomplete
    onKey: Incomplete
    def __init__(self, size=..., name: str = ...) -> None: ...
    def getSize(self): ...
    def isInteractive(self): ...
    def canUpdate(self): ...
    def clear(self) -> None: ...
    def flush(self) -> None: ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def setInfoLine(self, s) -> None: ...
    def stringBox(self, s, font: Incomplete | None = ...): ...
    def stringWidth(self, s, font: Incomplete | None = ...) -> None: ...
    def fontHeight(self, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...) -> None: ...
    def fontDescent(self, font: Incomplete | None = ...) -> None: ...
    def arcPoints(self, x1, y1, x2, y2, startAng: int = ..., extent: int = ...): ...
    def curvePoints(self, x1, y1, x2, y2, x3, y3, x4, y4): ...
    def drawMultiLineString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawLines(
        self,
        lineList,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def drawCurve(
        self,
        x1,
        y1,
        x2,
        y2,
        x3,
        y3,
        x4,
        y4,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawRect(
        self,
        x1,
        y1,
        x2,
        y2,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawRoundRect(
        self,
        x1,
        y1,
        x2,
        y2,
        rx: int = ...,
        ry: int = ...,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawEllipse(
        self,
        x1,
        y1,
        x2,
        y2,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawArc(
        self,
        x1,
        y1,
        x2,
        y2,
        startAng: int = ...,
        extent: int = ...,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawFigure(
        self,
        partList,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawImage(
        self,
        image,
        x1,
        y1,
        x2: Incomplete | None = ...,
        y2: Incomplete | None = ...,
        **kwargs
    ) -> None: ...

def getFileObject(file, openFlags: str = ...): ...

class AffineMatrix:
    A: Incomplete
    def __init__(self, init: Incomplete | None = ...) -> None: ...
    def scale(self, sx, sy) -> None: ...
    def rotate(self, theta) -> None: ...
    def translate(self, tx, ty) -> None: ...



from . import ClusterUtils as ClusterUtils
from _typeshed import Incomplete
from rdkit.sping import pid as pid

piddle = pid

class VisOpts:
    xOffset: int
    yOffset: int
    lineColor: Incomplete
    hideColor: Incomplete
    terminalColors: Incomplete
    lineWidth: int
    hideWidth: float
    nodeRad: int
    nodeColor: Incomplete
    highlightColor: Incomplete
    highlightRad: int

def _scaleMetric(val, power: int = ..., min: float = ...): ...

class ClusterRenderer:
    canvas: Incomplete
    size: Incomplete
    ptColors: Incomplete
    lineWidth: Incomplete
    showIndices: Incomplete
    showNodes: Incomplete
    stopAtCentroids: Incomplete
    logScale: Incomplete
    tooClose: Incomplete
    def __init__(
        self,
        canvas,
        size,
        ptColors=...,
        lineWidth: Incomplete | None = ...,
        showIndices: int = ...,
        showNodes: int = ...,
        stopAtCentroids: int = ...,
        logScale: int = ...,
        tooClose: int = ...,
    ) -> None: ...
    pts: Incomplete
    nPts: Incomplete
    xSpace: Incomplete
    def _AssignPointLocations(self, cluster, terminalOffset: int = ...) -> None: ...
    def _AssignClusterLocations(self, cluster) -> None: ...
    def _DrawToLimit(self, cluster) -> None: ...
    ySpace: Incomplete
    def DrawTree(self, cluster, minHeight: float = ...) -> None: ...

def DrawClusterTree(
    cluster,
    canvas,
    size,
    ptColors=...,
    lineWidth: Incomplete | None = ...,
    showIndices: int = ...,
    showNodes: int = ...,
    stopAtCentroids: int = ...,
    logScale: int = ...,
    tooClose: int = ...,
) -> None: ...
def _DrawClusterTree(
    cluster,
    canvas,
    size,
    ptColors=...,
    lineWidth: Incomplete | None = ...,
    showIndices: int = ...,
    showNodes: int = ...,
    stopAtCentroids: int = ...,
    logScale: int = ...,
    tooClose: int = ...,
) -> None: ...
def ClusterToPDF(
    cluster,
    fileName,
    size=...,
    ptColors=...,
    lineWidth: Incomplete | None = ...,
    showIndices: int = ...,
    stopAtCentroids: int = ...,
    logScale: int = ...,
): ...
def ClusterToSVG(
    cluster,
    fileName,
    size=...,
    ptColors=...,
    lineWidth: Incomplete | None = ...,
    showIndices: int = ...,
    stopAtCentroids: int = ...,
    logScale: int = ...,
): ...
def ClusterToImg(
    cluster,
    fileName,
    size=...,
    ptColors=...,
    lineWidth: Incomplete | None = ...,
    showIndices: int = ...,
    stopAtCentroids: int = ...,
    logScale: int = ...,
): ...



from typing import Any

class Snapshot:
    def __init__(self, RDKit): ...
    @classmethod
    def GetEnergy(cls, RDKit) -> Any: ...
    @classmethod
    def GetPoint2D(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetPoint3D(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetEnergy(cls, RDKit, double) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class Trajectory:
    def __init__(self, RDKit): ...
    @classmethod
    def AddConformersToMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def AddSnapshot(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Clear(cls, RDKit) -> Any: ...
    @classmethod
    def Dimension(cls, RDKit) -> Any: ...
    @classmethod
    def GetSnapshot(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def InsertSnapshot(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def NumPoints(cls, RDKit) -> Any: ...
    @classmethod
    def RemoveSnapshot(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def __len__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def ReadAmberTrajectory(*args, **kwargs) -> Any: ...
def ReadGromosTrajectory(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors

_smartsPatterns: Incomplete
_patternOrder: Incomplete
defaultPatternFileName: str

def _ReadPatts(fileName): ...

_GetAtomContribs: Incomplete

def _pyGetAtomContribs(
    mol,
    patts: Incomplete | None = ...,
    order: Incomplete | None = ...,
    verbose: int = ...,
    force: int = ...,
): ...
def _Init() -> None: ...
def _pyMolLogP(
    inMol,
    patts: Incomplete | None = ...,
    order: Incomplete | None = ...,
    verbose: int = ...,
    addHs: int = ...,
): ...
def _pyMolMR(
    inMol,
    patts: Incomplete | None = ...,
    order: Incomplete | None = ...,
    verbose: int = ...,
    addHs: int = ...,
): ...

MolLogP: Incomplete
MolMR: Incomplete



from _typeshed import Incomplete

numericTypes: Incomplete

class MLDataSet:
    data: Incomplete
    nResults: Incomplete
    nVars: Incomplete
    nPts: Incomplete
    qBounds: Incomplete
    nPossibleVals: Incomplete
    varNames: Incomplete
    ptNames: Incomplete
    def __init__(
        self,
        data,
        nVars: Incomplete | None = ...,
        nPts: Incomplete | None = ...,
        nPossibleVals: Incomplete | None = ...,
        qBounds: Incomplete | None = ...,
        varNames: Incomplete | None = ...,
        ptNames: Incomplete | None = ...,
        nResults: int = ...,
    ) -> None: ...
    def _CalcNPossible(self, data): ...
    def GetNResults(self): ...
    def GetNVars(self): ...
    def GetNPts(self): ...
    def GetNPossibleVals(self): ...
    def GetQuantBounds(self): ...
    def __getitem__(self, idx): ...
    def __setitem__(self, idx, val): ...
    def GetNamedData(self): ...
    def GetAllData(self): ...
    def GetInputData(self): ...
    def GetResults(self): ...
    def GetVarNames(self): ...
    def GetPtNames(self): ...
    def AddPoint(self, pt) -> None: ...
    def AddPoints(self, pts, names) -> None: ...

class MLQuantDataSet(MLDataSet):
    def _CalcNPossible(self, data): ...
    def GetNamedData(self): ...
    def GetAllData(self): ...
    def GetInputData(self): ...
    def GetResults(self): ...
    data: Incomplete
    nResults: Incomplete
    nVars: Incomplete
    nPts: Incomplete
    qBounds: Incomplete
    nPossibleVals: Incomplete
    varNames: Incomplete
    ptNames: Incomplete
    def __init__(
        self,
        data,
        nVars: Incomplete | None = ...,
        nPts: Incomplete | None = ...,
        nPossibleVals: Incomplete | None = ...,
        qBounds: Incomplete | None = ...,
        varNames: Incomplete | None = ...,
        ptNames: Incomplete | None = ...,
        nResults: int = ...,
    ) -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

aldehydePatt: Incomplete
ketonePatt: Incomplete
amidePatt: Incomplete
esterPatt: Incomplete
carboxylatePatt: Incomplete
carboxylPatt: Incomplete
specialCases: Incomplete

def SATISTypes(mol, neighborsToInclude: int = ...): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

log: Incomplete

class MetalDisconnector:
    _metal_nof: Incomplete
    _metal_non: Incomplete
    def __init__(self) -> None: ...
    def __call__(self, mol): ...
    def disconnect(self, mol): ...



from rdkit import Chem as Chem

def GetPrincipleQuantumNumber(atNum): ...
def EStateIndices(mol, force: bool = ...): ...
def MaxEStateIndex(mol, force: int = ...): ...
def MinEStateIndex(mol, force: int = ...): ...
def MaxAbsEStateIndex(mol, force: int = ...): ...
def MinAbsEStateIndex(mol, force: int = ...): ...
def _exampleCode() -> None: ...



from _typeshed import Incomplete

cdxModule: Incomplete
_cdxVersion: int
cdApp: Incomplete
theDoc: Incomplete
theObjs: Incomplete
selectItem: Incomplete
cleanItem: Incomplete
centerItem: Incomplete

def StartChemDraw(
    visible: bool = ..., openDoc: bool = ..., showDoc: bool = ...
) -> None: ...
def ReactivateChemDraw(openDoc: bool = ..., showDoc: bool = ...) -> None: ...
def CDXConvert(inData, inFormat, outFormat): ...
def CDXClean(inData, inFormat, outFormat): ...
def CDXDisplay(inData, inFormat: str = ..., clear: int = ...) -> None: ...
def CDXGrab(outFormat: str = ...): ...
def CloseChemdraw() -> None: ...
def Exit() -> None: ...
def SaveChemDrawDoc(fileName: str = ...) -> None: ...
def CloseChemDrawDoc() -> None: ...
def RaiseWindowNamed(nameRe): ...
def RaiseChemDraw() -> None: ...
def SmilesToPilImage(smilesStr): ...
def MolToPilImage(dataStr, inFormat: str = ..., outFormat: str = ...): ...

c3dApp: Incomplete

def StartChem3D(visible: int = ...) -> None: ...
def CloseChem3D() -> None: ...

availChem3DProps: Incomplete

def Add3DCoordsToMol(data, format, props=...): ...
def OptimizeSDFile(
    inFileName, outFileName, problemFileName: str = ..., restartEvery: int = ...
) -> None: ...



from _typeshed import Incomplete

def _CheckCombArgument(n: int, k: int) -> None: ...

nPointDistDict: Incomplete
nDistPointDict: Incomplete
_trianglesInPharmacophore: Incomplete

def GetTriangles(nPts): ...
def BinsTriangleInequality(d1, d2, d3): ...
def ScaffoldPasses(combo, bins: Incomplete | None = ...): ...

_numCombDict: Incomplete

def NumCombinations(nItems, nSlots): ...

_verbose: int
_countCache: Incomplete

def CountUpTo(nItems, nSlots, vs, idx: int = ..., startAt: int = ...): ...

_indexCombinations: Incomplete

def GetIndexCombinations(nItems, nSlots, slot: int = ..., lastItemVal: int = ...): ...
def GetAllCombinations(choices, noDups: int = ..., which: int = ...): ...
def GetUniqueCombinations(choices, classes, which: int = ...): ...
def GetUniqueCombinations_new(choices, classes, which: int = ...): ...
def UniquifyCombinations(combos): ...
def GetPossibleScaffolds(nPts, bins, useTriangleInequality: bool = ...): ...
def OrderTriangle(featIndices, dists): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.ML.Cluster import Clusters as Clusters
from rdkit.ML.Cluster.Clustering import (
    MurtaghCluster as MurtaghCluster,
    MurtaghDistCluster as MurtaghDistCluster,
)

WARDS: int
SLINK: int
CLINK: int
UPGMA: int
MCQUITTY: int
GOWER: int
CENTROID: int
methods: Incomplete

def _LookupDist(dists, i, j, n): ...
def _ToClusters(data, nPts, ia, ib, crit, isDistData: int = ...): ...
def ClusterData(data, nPts, method, isDistData: int = ...): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig
from rdkit.Avalon import pyAvalonTools as pyAvalonTools
from rdkit.Chem.MolKey import InchiInfo as InchiInfo
from typing import NamedTuple

class MolIdentifierException(Exception): ...
class BadMoleculeException(Exception): ...

MOL_KEY_VERSION: str
ERROR_DICT: Incomplete
INCHI_COMPUTATION_ERROR: Incomplete
RDKIT_CONVERSION_ERROR: Incomplete
INCHI_READWRITE_ERROR: Incomplete
NULL_MOL: Incomplete
BAD_SET: Incomplete
GET_STEREO_RE: Incomplete
NULL_SMILES_RE: Incomplete
PATTERN_NULL_MOL: str
CHIRAL_POS: int
T_NULL_MOL: Incomplete
stereo_code_dict: Incomplete

def _fix_all(pat, sbt, my_string): ...
def _fix_line_ends(my_string): ...
def _fix_chemdraw_header(my_string): ...
def _ctab_has_atoms(ctab_lines): ...
def _ctab_remove_chiral_flag(ctab_lines): ...

__initCalled: bool

def initStruchk(
    configDir: Incomplete | None = ..., logFile: Incomplete | None = ...
) -> None: ...
def CheckCTAB(ctab, isSmiles: bool = ...): ...

class InchiResult(NamedTuple):
    error: Incomplete
    inchi: Incomplete
    fixed_ctab: Incomplete

def GetInchiForCTAB(ctab): ...
def _make_racemate_inchi(inchi): ...
def _get_identification_string(
    err,
    ctab,
    inchi,
    stereo_category: Incomplete | None = ...,
    extra_stereo: Incomplete | None = ...,
): ...
def _get_null_mol_identification_string(extra_stereo): ...
def _get_bad_mol_identification_string(ctab, stereo_category, extra_stereo): ...
def _identify(
    err, ctab, inchi, stereo_category, extra_structure_desc: Incomplete | None = ...
): ...
def _get_chiral_identification_string(n_def, n_udf): ...
def ErrorBitsToText(err): ...

class MolKeyResult(NamedTuple):
    mol_key: Incomplete
    error: Incomplete
    inchi: Incomplete
    fixed_ctab: Incomplete
    stereo_code: Incomplete
    stereo_comment: Incomplete

def GetKeyForCTAB(
    ctab,
    stereo_info: Incomplete | None = ...,
    stereo_comment: Incomplete | None = ...,
    logger: Incomplete | None = ...,
): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs

class BitClusterer:
    _clusters: Incomplete
    _bidList: Incomplete
    _nClusters: Incomplete
    _type: Incomplete
    def __init__(self, idList, nCluster, type=...) -> None: ...
    def ClusterBits(self, corrMat) -> None: ...
    def SetClusters(self, clusters) -> None: ...
    def GetClusters(self): ...
    def MapToClusterScores(self, fp): ...
    def MapToClusterFP(self, fp): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import (
    AllChem as AllChem,
    Crippen as Crippen,
    Descriptors as Descriptors,
    Lipinski as Lipinski,
)
from rdkit.Dbase import DbModule as DbModule
from rdkit.Dbase.DbConnection import DbConnect as DbConnect

logger: Incomplete

def ProcessMol(
    mol,
    typeConversions,
    globalProps,
    nDone,
    nameProp: str = ...,
    nameCol: str = ...,
    redraw: bool = ...,
    keepHs: bool = ...,
    skipProps: bool = ...,
    addComputedProps: bool = ...,
    skipSmiles: bool = ...,
    uniqNames: Incomplete | None = ...,
    namesSeen: Incomplete | None = ...,
): ...
def ConvertRows(rows, globalProps, defaultVal, skipSmiles) -> None: ...
def LoadDb(
    suppl,
    dbName,
    nameProp: str = ...,
    nameCol: str = ...,
    silent: bool = ...,
    redraw: bool = ...,
    errorsTo: Incomplete | None = ...,
    keepHs: bool = ...,
    defaultVal: str = ...,
    skipProps: bool = ...,
    regName: str = ...,
    skipSmiles: bool = ...,
    maxRowsCached: int = ...,
    uniqNames: bool = ...,
    addComputedProps: bool = ...,
    lazySupplier: bool = ...,
    startAnew: bool = ...,
) -> None: ...



from .errors import (
    MolVSError as MolVSError,
    StandardizeError as StandardizeError,
    ValidateError as ValidateError,
)
from .standardize import (
    Standardizer as Standardizer,
    canonicalize_tautomer_smiles as canonicalize_tautomer_smiles,
    enumerate_tautomers_smiles as enumerate_tautomers_smiles,
    standardize_smiles as standardize_smiles,
)
from .validate import Validator as Validator, validate_smiles as validate_smiles
from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem.MolStandardize import rdMolStandardize as rdMolStandardize

__title__: str
__version__: str
__author__: str
__email__: str
__license__: str
__copyright__: str
log: Incomplete

def ReorderTautomers(molecule): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig
from rdkit.Chem.Suppliers import DbMolSupplier as DbMolSupplier
from rdkit.VLib.Supply import SupplyNode as SupplyNode

class DbMolSupplyNode(SupplyNode):
    _dbResults: Incomplete
    _supplier: Incomplete
    def __init__(self, dbResults, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def next(self): ...

def GetNode(dbName, tableName): ...
def _test(): ...
